@@ -1,4 +1,4 @@
-/*	$Id: vilter-spamd.c,v 1.25 2006/02/09 08:48:53 mbalmer Exp $	*/
+/*	$Id: vilter-spamd.c,v 1.30 2006/02/16 08:50:56 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -56,7 +56,7 @@ int	 spamd_timeout;
 off_t	 spamd_maxsize;
 
 int
-vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double *threshold)
+vilter_scan(char *fn, size_t fnlen, char *chroot, double *score, double *threshold)
 {
 	int fd;
 	int try;
@@ -87,7 +87,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 		return SCAN_ERROR;
 	}
 	if (bind_addr != NULL) {
-		bzero((char *) &server_sockaddr, sizeof(server_sockaddr));
+		bzero((char *)&server_sockaddr, sizeof(server_sockaddr));
 		bind_sockaddr.sin_family = AF_INET;
 
 		if (inet_aton(bind_addr, &bind_sockaddr.sin_addr) == 0) {
@@ -100,13 +100,13 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 			bind_sockaddr.sin_family = hostent->h_addrtype;
 			memcpy(&bind_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
 		}
-		if (bind(fd, (struct sockaddr *) &bind_sockaddr, sizeof(bind_sockaddr))) {
+		if (bind(fd, (struct sockaddr *)&bind_sockaddr, sizeof(bind_sockaddr))) {
 			syslog(LOG_ERR, "spam: can't bind to address %s", bind_addr);
 			close(fd);
 			return SCAN_ERROR;
 		}
 	}
-	bzero((char *) &server_sockaddr, sizeof(server_sockaddr));
+	bzero((char *)&server_sockaddr, sizeof(server_sockaddr));
 
 	server_sockaddr.sin_family = AF_INET;
 	server_sockaddr.sin_port = htons(spamd_port);
@@ -123,7 +123,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 	}
 	try = 0;
 
-	while (connect(fd, (struct sockaddr *) &server_sockaddr, sizeof(server_sockaddr)) == -1) {
+	while (connect(fd, (struct sockaddr *)&server_sockaddr, sizeof(server_sockaddr)) == -1) {
 		++try;
 
 		if (try == spamd_tries) {
@@ -141,7 +141,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 		to = NULL;
 
 	snprintf(buf, sizeof(buf), "CHECK SPAMC/1.2\r\nContent-length: %lld\r\n\r\n",
-		(long long) statbuf.st_size);
+		(long long)statbuf.st_size);
 	buflen = strlen(buf);
 		
 	if (to_send(fd, buf, buflen, 0, to) != buflen) {
@@ -169,7 +169,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 		return SCAN_ERROR;
 	}
 	if ((sp = fdsock(fd)) == NULL) {
-		syslog(LOG_ERR, "spamd: error buffering socker");
+		syslog(LOG_ERR, "spamd: error buffering socket");
 		close(fd);
 		return SCAN_ERROR;
 	}
@@ -208,7 +208,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, double *score, double
 char *
 vilter_name(void)
 {
-	return "Spamassassin Daemon (spamd)";
+	return "SpamAssassin Daemon (spamd)";
 }
 
 int
