@@ -1,4 +1,4 @@
-/*	$Id: engine.c,v 1.37 2006/02/08 14:47:28 mbalmer Exp $	*/
+/*	$Id: engine.c,v 1.48 2006/02/17 07:33:14 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>.
@@ -34,7 +34,6 @@
 #include <math.h>
 #include <pthread.h>
 #include <pwd.h>
-#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -91,6 +90,7 @@ extern char	*clean_table;
 extern long	 clean_duration;
 
 extern int	 markall;
+extern int	 logall;
 extern int	 verbose;
 extern char	*pidfile;
 extern char	*cfgfile;
@@ -112,15 +112,9 @@ extern int	 nbe;
 extern char	*spamprefix;
 extern int	 logvirus, logspam, logunwanted;
 extern int	 keep_tmpfiles;
-extern int	 ex_timeout;
 
 struct imsgbuf	*ibuf_main;
 
-/* For ceonnection reference counting */
-
-static int		conn_cnt = 0;
-static pthread_mutex_t	conn_cnt_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 extern char *__progname;
 
 #ifdef ENABLE_LDAP
@@ -192,7 +186,6 @@ struct message {
 struct privdata {
 	struct connection	*conn;
 	struct message		*msg;
-	struct be_data		*backend_data;
 };
 
 #ifdef ENABLE_PF
@@ -226,6 +219,8 @@ get_v_strategy(const char *s)
 		return STRATEGY_NOTIFY_RECIPIENT;
 	else if (!strcmp(s, "reject"))
 		return STRATEGY_REJECT;
+	else if (!strcmp(s, "ignore"))
+		return STRATEGY_IGNORE;
 	else
 		return STRATEGY_UNSET;
 }
@@ -239,6 +234,8 @@ get_su_strategy(const char *s)
 		return STRATEGY_NOTIFY_RECIPIENT;
 	else if (!strcmp(s, "reject"))
 		return STRATEGY_REJECT;
+	else if (!strcmp(s, "ignore"))
+		return STRATEGY_IGNORE;
 	else
 		return STRATEGY_UNSET;
 }
@@ -270,29 +267,13 @@ connection_setup(SMFICTX *ctx)
 
 	priv->conn = conn;
 
-	if ((priv->backend_data = malloc(sizeof(struct be_data) * nbe)) == NULL) {
-		syslog(LOG_ERR, "unable to allocate memory for backend data");
-		return SMFIS_TEMPFAIL;
-	}
-	bzero(priv->backend_data, sizeof(struct be_data) * nbe);
-
 	if (smfi_setpriv(ctx, priv)) {
 		syslog(LOG_ERR, "unable to store internal data");
 		free(priv->conn);
-		free(priv->backend_data);
 		free(priv);
 		return SMFIS_TEMPFAIL;
 	}
 
-	if (pthread_mutex_lock(&conn_cnt_mutex))
-		syslog(LOG_ERR, "unable to increase connection count");
-	else {
-		++conn_cnt;
-		if (verbose)
-			warnx("connection increased to %d", conn_cnt);
-		pthread_mutex_unlock(&conn_cnt_mutex);
-	}
-
 	return SMFIS_CONTINUE;
 }
 
@@ -393,7 +374,7 @@ connection_cleanup(SMFICTX *ctx)
 {
 	struct privdata		*priv;
 	struct connection	*conn;
-	
+
 	if (verbose)
 		warnx("connection cleanup");
 
@@ -408,24 +389,11 @@ connection_cleanup(SMFICTX *ctx)
 				free(conn->heloname);
 			free(conn);
 		}
-		if (priv->backend_data != NULL)
-			free(priv->backend_data);
 
 		free(priv);
 		smfi_setpriv(ctx, NULL);
 	} else
 		syslog(LOG_ERR, "unable to get internal data in connection_cleanup()");
-
-	if (pthread_mutex_lock(&conn_cnt_mutex))
-		syslog(LOG_ERR, "can't decrease connection count");
-	else {
-		--conn_cnt;
-		if (verbose)
-			warnx("connection count decreased to %d", conn_cnt);
-		if (conn_cnt < 0)
-			syslog(LOG_ERR, "connection count drops below zero");
-		pthread_mutex_unlock(&conn_cnt_mutex);
-	}
 }
 
 static sfsistat
@@ -539,10 +507,11 @@ smtp_vilter_envfrom(SMFICTX *ctx, char **argv)
 			unlink(msg->tfn);
 		}
 		return SMFIS_TEMPFAIL;
-	}		
+	}
+
 	for (n = 0; n < nbe; n++) {
-		if (be[n].be_new != NULL) {
-			if (be[n].be_new(&priv->backend_data[n], argv[0], msg->fp)) {
+		if (be[n].be_envfrom != NULL) {
+			if (be[n].be_envfrom(argv[0], msg->fp)) {
 				syslog(LOG_ERR, "vilter-engine refuses to work on this message");
 				/* vilter_cleanup(ctx); */
 				return SMFIS_TEMPFAIL;
@@ -938,7 +907,7 @@ log_incident(struct privdata *priv, char *type, char *backend, char *value)
 }
 
 static sfsistat
-vilter_virus(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx, struct privdata *priv, int *stop_chain, int *isvirus)
+vilter_virus(struct backend *backend, SMFICTX *ctx, struct privdata *priv, int *stop_chain, int *isvirus)
 {
 	int			 retval = SMFIS_CONTINUE;
 	char			 virus[MAXLEN];
@@ -953,7 +922,7 @@ vilter_virus(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx
 	/* Scan for virus */
 
 	strlcpy(scanfn, msg->tfn, sizeof(scanfn));
-	scanresult = backend->be_scan.be_scan_virus(backend_data, scanfn, chrootdir, virus, sizeof(virus));
+	scanresult = backend->be_scan.be_scan_virus(scanfn, sizeof(scanfn), chrootdir, virus, sizeof(virus));
 
 	switch (scanresult) {
 	case SCAN_VIRUS:
@@ -969,6 +938,10 @@ vilter_virus(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx
 			log_incident(priv, "virus", backend->name, virus);
 
 		switch (virus_strategy) {
+		case STRATEGY_IGNORE:
+			syslog(LOG_INFO, "ignore it and deliver");
+			retval = SMFIS_CONTINUE;
+			break;
 		case STRATEGY_MARK:
 			syslog(LOG_INFO, "marking message (but deliver anyway)");
 			mark_virus(ctx, "infected", virus, backend->name);
@@ -1016,7 +989,8 @@ vilter_virus(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx
 		*isvirus = 1;	
 		break;
 	case SCAN_OK:
-		syslog(LOG_INFO, "message contains no virus");
+		if (logall)
+			syslog(LOG_INFO, "message contains no virus");
 
 		if (markall)
 			mark_virus(ctx, "clean", NULL, backend->name);
@@ -1060,7 +1034,7 @@ vilter_virus(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx
 }
 
 static sfsistat
-vilter_spam(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx, struct privdata *priv, int *spam)
+vilter_spam(struct backend *backend, SMFICTX *ctx, struct privdata *priv, int *spam)
 {
 	int			 retval = SMFIS_CONTINUE;
 	int			 scanresult;
@@ -1068,13 +1042,15 @@ vilter_spam(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx,
 	char			*subject;
 	unsigned int		 len;
 	struct message		*msg;
+	char			 scanfn[PATH_MAX];
 
 	msg = priv->msg;
 	score = threshold = 0.0;
 
 	/* Scan for spam */
 
-	scanresult = backend->be_scan.be_scan_spam(backend_data, msg->tfn, chrootdir, &score, &threshold);
+	strlcpy(scanfn, msg->tfn, sizeof(scanfn));
+	scanresult = backend->be_scan.be_scan_spam(scanfn, sizeof(scanfn), chrootdir, &score, &threshold);
 
 	switch (scanresult) {
 	case SCAN_SPAM:
@@ -1089,6 +1065,10 @@ vilter_spam(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx,
 			log_incident(priv, "spam", backend->name, NULL);
 
 		switch (spam_strategy) {
+		case STRATEGY_IGNORE:
+			syslog(LOG_INFO, "ignore it and deliver");
+			retval = SMFIS_CONTINUE;
+			break;
 		case STRATEGY_DISCARD:
 			syslog(LOG_INFO, "discarding message");
 			retval = SMFIS_DISCARD;
@@ -1143,8 +1123,11 @@ vilter_spam(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx,
 		*spam = 1;
 		break;
 	case SCAN_OK:
-		syslog(LOG_INFO, "message is not spam");
-		mark_spam(ctx, NULL, backend->name, score, threshold);
+		if (logall)
+			syslog(LOG_INFO, "message is not spam");
+
+		if (markall)
+			mark_spam(ctx, NULL, backend->name, score, threshold);
 		break;
 	case SCAN_ERROR:
 		syslog(LOG_ERR, "error during spam scan of file %s", msg->tfn);
@@ -1175,7 +1158,7 @@ vilter_spam(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx,
 }
 
 static sfsistat
-vilter_unwanted(struct backend *backend, struct be_data *backend_data, SMFICTX *ctx, struct privdata *priv, int *unwanted)
+vilter_unwanted(struct backend *backend, SMFICTX *ctx, struct privdata *priv, int *unwanted)
 {
 	int			 retval = SMFIS_CONTINUE;
 	char			 reason[MAXLEN];
@@ -1190,7 +1173,7 @@ vilter_unwanted(struct backend *backend, struct be_data *backend_data, SMFICTX *
 	/* Scan for unwanted content */
 
 	strlcpy(scanfn, msg->tfn, sizeof(scanfn));
-	scanresult = backend->be_scan.be_scan_unwanted(backend_data, scanfn, chrootdir, reason, sizeof(reason));
+	scanresult = backend->be_scan.be_scan_unwanted(scanfn, sizeof(scanfn), chrootdir, reason, sizeof(reason));
 
 	switch (scanresult) {
 	case SCAN_UNWANTED:
@@ -1205,6 +1188,10 @@ vilter_unwanted(struct backend *backend, struct be_data *backend_data, SMFICTX *
 			log_incident(priv, "unwanted-content", backend->name, reason);
 
 		switch (unwanted_strategy) {
+		case STRATEGY_IGNORE:
+			syslog(LOG_INFO, "ignore it and deliver");
+			retval = SMFIS_CONTINUE;
+			break;
 		case STRATEGY_MARK:
 			syslog(LOG_INFO, "marking message (but deliver anyway)");
 			mark_unwanted(ctx, "unwanted content", reason, backend->name);
@@ -1249,7 +1236,6 @@ vilter_unwanted(struct backend *backend, struct be_data *backend_data, SMFICTX *
 		*unwanted = 1;
 		break;
 	case SCAN_OK:
-		syslog(LOG_INFO, "message contains no unwanted content");
 		if (strcmp(scanfn, msg->tfn)) {
 			if (markall)
 				mark_unwanted(ctx, "cleaned", NULL, backend->name);
@@ -1265,8 +1251,12 @@ vilter_unwanted(struct backend *backend, struct be_data *backend_data, SMFICTX *
 			if (unlink(scanfn))
 				syslog(LOG_ERR, "can't delete temp file %s", scanfn);
 
-		} else if (markall)
-			mark_unwanted(ctx, "clean", NULL, backend->name);
+		} else {
+			if (logall)
+				syslog(LOG_INFO, "message contains no unwanted content");
+			if (markall)
+				mark_unwanted(ctx, "clean", NULL, backend->name);
+		}
 		break;
 	case SCAN_ERROR:
 		syslog(LOG_ERR, "error during scan of file %s", msg->tfn);
@@ -1308,13 +1298,14 @@ print_entries_with_values(LDAP *ld, LDAPMessage *result)
 
 	for (e = ldap_first_entry(ld, result); e != NULL; e = ldap_next_entry(ld, e)) {
 		if ((dn = ldap_get_dn(ld, e)) != NULL) {
-			warnx("dn: %s", dn);
+			if (verbose)
+				warnx("dn: %s", dn);
 			ldap_memfree(dn);
 		}
 		for (attr = ldap_first_attribute(ld, e, &ber); attr != NULL; attr = ldap_next_attribute(ld, e, ber)) {
 			if ((vals = ldap_get_values(ld, e, attr)) != NULL) {
 				for (i = 0; vals[i] != NULL; i++)
-					warnx("%s: %s", attr, vals[i]);
+					printf("%s: %s\n", attr, vals[i]);
 				ldap_value_free(vals);
 			}
 			ldap_memfree(attr);
@@ -1332,7 +1323,6 @@ vilter_eom(SMFICTX *ctx)
 	struct		 message *msg;
 	int		 retval = SMFIS_CONTINUE;
 	int		 n;
-	struct		 be_data *backend_data;
 	int		 stop_chain;
 	int		 status, allstat;
 #ifdef ENABLE_LDAP
@@ -1375,21 +1365,20 @@ vilter_eom(SMFICTX *ctx)
 
 		allstat = 0;
 		for (n = stop_chain = 0; n < nbe && !stop_chain && retval == SMFIS_CONTINUE; n++) {
-			backend_data = &priv->backend_data[n];
 			status = 0;
 			if (be[n].type & BE_SCAN_VIRUS) {
 				if (verbose)
 					warnx("scanning for viruses using backend %s", be[n].name);
-				retval = vilter_virus(&be[n], backend_data, ctx, priv, &stop_chain, &status);
+				retval = vilter_virus(&be[n], ctx, priv, &stop_chain, &status);
 			} else if (be[n].type & BE_SCAN_SPAM) {
 				if (verbose)
 					warnx("scanning for spam using backend %s", be[n].name);
 
-				retval = vilter_spam(&be[n], backend_data, ctx, priv, &status);
+				retval = vilter_spam(&be[n], ctx, priv, &status);
 			} else if (be[n].type & BE_SCAN_UNWANTED) {
 				if (verbose)
 					warnx("scanning for unwanted content using backend %s", be[n].name);
-				retval = vilter_unwanted(&be[n], backend_data, ctx, priv, &status);
+				retval = vilter_unwanted(&be[n], ctx, priv, &status);
 			} else
 				return SMFIS_TEMPFAIL;
 			allstat += status;
@@ -1411,7 +1400,8 @@ vilter_eom(SMFICTX *ctx)
 		}
 #ifdef ENABLE_LDAP
 		SLIST_FOREACH(r, &msg->rhead, entries) {
-			warnx("recipient: %s (domain %s)", r->mailLocalAddress, r->domain->mailLocalDomain);
+			if (verbose)
+				warnx("recipient: %s (domain %s)", r->mailLocalAddress, r->domain->mailLocalDomain);
 			nentries = 0;
 			while ((rc = ldap_result(ld, r->msgid, 0, NULL, &result)) == LDAP_RES_SEARCH_ENTRY) {
 				nentries++;
@@ -1420,19 +1410,19 @@ vilter_eom(SMFICTX *ctx)
 				for (e = ldap_first_entry(ld, result); e != NULL; e = ldap_next_entry(ld, e)) {
 					if ((vals = ldap_get_values(ld, e, "spamStrategy")) != NULL) {
 						r->spam_strategy = get_su_strategy(vals[0]);
-						if (r->spam_strategy == STRATEGY_UNSET)
+						if (r->spam_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown spam strategy '%s' for recipient '%s'", vals[0], r->mailLocalAddress);
 						ldap_value_free(vals);
 					}
 					if ((vals = ldap_get_values(ld, e, "virusStrategy")) != NULL) {
 						r->virus_strategy = get_v_strategy(vals[0]);
-						if (r->virus_strategy == STRATEGY_UNSET)
+						if (r->virus_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown virus strategy '%s' for recipient '%s'", vals[0], r->mailLocalAddress);
 						ldap_value_free(vals);
 					}
 					if ((vals = ldap_get_values(ld, e, "unwantedStrategy")) != NULL) {
 						r->unwanted_strategy = get_su_strategy(vals[0]);
-						if (r->unwanted_strategy == STRATEGY_UNSET)
+						if (r->unwanted_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown unwanted strategy '%s' for recipient '%s'", vals[0], r->mailLocalAddress);
 						ldap_value_free(vals);
 					}
@@ -1445,7 +1435,8 @@ vilter_eom(SMFICTX *ctx)
 			}
 		}
 		SLIST_FOREACH(d, &msg->dhead, entries) {
-			warnx("domain: %s", d->mailLocalDomain);
+			if (verbose)
+				warnx("domain: %s", d->mailLocalDomain);
 			nentries = 0;
 			while ((rc = ldap_result(ld, d->msgid, 0, NULL, &result)) == LDAP_RES_SEARCH_ENTRY) {
 				nentries++;
@@ -1454,19 +1445,19 @@ vilter_eom(SMFICTX *ctx)
 				for (e = ldap_first_entry(ld, result); e != NULL; e = ldap_next_entry(ld, e)) {
 					if ((vals = ldap_get_values(ld, e, "spamStrategy")) != NULL) {
 						d->spam_strategy = get_su_strategy(vals[0]);
-						if (d->spam_strategy == STRATEGY_UNSET)
+						if (d->spam_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown spam strategy '%s' for recipient domain '%s'", vals[0], d->mailLocalDomain);
 						ldap_value_free(vals);
 					}
 					if ((vals = ldap_get_values(ld, e, "virusStrategy")) != NULL) {
 						d->virus_strategy = get_v_strategy(vals[0]);
-						if (d->virus_strategy == STRATEGY_UNSET)
+						if (d->virus_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown virus strategy '%s' for recipient domain '%s'", vals[0], d->mailLocalDomain);
 						ldap_value_free(vals);
 					}
 					if ((vals = ldap_get_values(ld, e, "unwantedStrategy")) != NULL) {
 						d->unwanted_strategy = get_su_strategy(vals[0]);
-						if (d->unwanted_strategy == STRATEGY_UNSET)
+						if (d->unwanted_strategy == STRATEGY_UNSET && verbose)
 							warnx("unknown unwanted strategy '%s' for recipient domain '%s'", vals[0], d->mailLocalDomain);
 						ldap_value_free(vals);
 					}
@@ -1530,11 +1521,13 @@ vilter_close(SMFICTX *ctx)
 		if ((priv = smfi_getpriv(ctx)) != NULL) {
 			conn = priv->conn;
 			if (conn != NULL && conn->hostname != NULL) {
-				warnx("closing connection with %s", conn->hostname);
+				if (verbose)
+					warnx("closing connection with %s", conn->hostname);
 				goto cleanup;
 			}
 		}
-		warnx("closing connection");
+		if (verbose)
+			warnx("closing connection");
 	}
 cleanup:
 	connection_cleanup(ctx);
@@ -1613,10 +1606,7 @@ e_main(uid_t uid, gid_t gid, int pipe_m2e[2])
 	smfiDesc_str	 desc;
 	FILE		*fp;
 	struct rlimit	 rl;
-	int		 n;
 	int		 pid;
-	int		 nconn;
-	int		 tout;
 
 	switch (pid = fork()) {
 	case -1:
@@ -1714,39 +1704,9 @@ e_main(uid_t uid, gid_t gid, int pipe_m2e[2])
 
 	smfi_setconn(port);
 	if (smfi_main() == MI_FAILURE)
-		syslog(LOG_ERR, "failed to establish a connection on port %s", port);
-
-	/* Wait for the connection reference count to drop to zero */
-
-	if (verbose)
-		warnx("gracefully terminating");
-
-	if (ex_timeout > 0)
-		tout = ex_timeout;
-
-	nconn = 0;
-	do {
-		if (pthread_mutex_lock(&conn_cnt_mutex))
-			syslog(LOG_ERR, "can get connection count");
-		else {
-			nconn = conn_cnt;
-			pthread_mutex_unlock(&conn_cnt_mutex);
-		}
-
-		if (nconn > 0) {
-			if (verbose)
-				warnx("waiting for connection count to reach zero (now %d)", nconn);
-			sleep(1);
-			if (ex_timeout > 0) {
-				if (tout-- == 0)
-					nconn = 0;
-			}
-		}
-	} while (nconn > 0);
-
-	if (verbose)
-		warnx("connection count reached zero, terminating instance");
+		syslog(LOG_ERR, "child process terminating");
 
+	/*
 	for (n = 0; n < nbe; n++) {
 		if (be[n].be_exit != NULL)
 			be[n].be_exit();
@@ -1754,8 +1714,7 @@ e_main(uid_t uid, gid_t gid, int pipe_m2e[2])
 		dlclose(be[n].dlhandle);
 	}
 	free(be);
-
-	syslog(LOG_INFO, "child process exiting");
+	*/
 
 	closelog();
 	if (logfp != NULL)
