@@ -1,4 +1,4 @@
-/*	$Id: pftable.c,v 1.19 2006/02/07 19:33:26 mbalmer Exp $ */
+/*	$Id: pftable.c,v 1.20 2006/02/17 07:38:32 mbalmer Exp $ */
 
 /*
  * Copyright (c) 2005, 2006 Marc Balmer <marc@msys.ch>.
@@ -43,7 +43,7 @@ struct timed_entry {
 	char				*pftable;
 	char				*host;
 	time_t				 expiry;
-	int				 what;
+	unsigned long			 what;
 	SLIST_ENTRY(timed_entry)	 entries;
 };
 
@@ -77,7 +77,7 @@ pftable_exists(const char *name)
 
 /* imsg handlers */
 static int
-pftable_modify(char *pftable, char *host, int what)
+pftable_modify(char *pftable, char *host, unsigned long what)
 {
 	struct hostent *h;
 	struct pfioc_table tio;
