@@ -1,4 +1,4 @@
-/*	$Id: smtp-vilter.c,v 1.143 2006/02/09 11:17:39 mbalmer Exp $	*/
+/*	$Id: smtp-vilter.c,v 1.151 2006/02/17 07:33:14 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -94,6 +94,7 @@ char	*unwanted_table;
 char	*clean_table;
 
 int	 markall;
+int	 logall;
 int	 verbose;
 char	*pidfile;
 char	*cfgfile;
@@ -134,8 +135,6 @@ char	*binddn;
 char	*bindpasswd;
 int	 ldap_use_tls;
 
-int	 ex_timeout;	/* Exit timeout waiting for pending connections */
-
 volatile sig_atomic_t quit = 0;
 volatile sig_atomic_t child_quit = 0;
 volatile sig_atomic_t reconfig = 0;
@@ -146,7 +145,7 @@ int	 dev = -1;
 struct privdata {
 	struct connection	*conn;
 	struct message		*msg;
-	struct be_data		*backend_data;
+	void			*backend_data;
 };
 
 int dispatch_imsg(struct imsgbuf *, int);
@@ -192,7 +191,6 @@ usage(void)
 #ifdef ENABLE_LDAP
 	    " [-D binddn]"
 #endif
-	    " [-E timeout]"
 	    " [-e logile]"
 	    " [-f maxfiles]"
 	    " [-g group]"
@@ -257,7 +255,8 @@ decode_backend(char *s)
 static int
 get_rebind_credentials(LDAP *ld, char **dnp, char **pwp, int *authmethodp, int freeit)
 {
-	warnx("get_rebind_credentials() called");
+	if (verbose)
+		warnx("get_rebind_credentials() called");
 	return LDAP_SUCCESS;
 }
 #endif
@@ -319,12 +318,12 @@ main(int argc, char *argv[])
 	recipient_notification = NULL;
 	notify_only = NULL;
 	markall = 0;
+	logall = 0;
 	rlimit_nofile = 0;
 	rlimit_nproc = 0;
 	spamprefix = NULL;
 	logvirus = logspam = logunwanted = 0;
 	keep_tmpfiles = 0;
-	ex_timeout = -1;
 #ifdef ENABLE_LDAP
 	ldaphost = NULL;
 	ldapport = -1;
@@ -339,9 +338,9 @@ main(int argc, char *argv[])
 	/* Process the commandline */
 
 #ifdef ENABLE_LDAP	
-	while ((ch = getopt(argc, argv, "A:a:B:b:C:D:d:E:e:f:g:h:kL:mn:o:P:p:T:t:u:U:Vvw:x?")) != -1) {
+	while ((ch = getopt(argc, argv, "A:a:B:b:C:D:d:e:f:g:h:kL:mn:o:P:p:T:t:u:U:Vvw:x?")) != -1) {
 #else
-	while ((ch = getopt(argc, argv, "A:a:b:C:d:E:e:f:g:kmn:o:P:p:T:t:u:Vvx?")) != -1) {
+	while ((ch = getopt(argc, argv, "A:a:b:C:d:e:f:g:kmn:o:P:p:T:t:u:Vvx?")) != -1) {
 #endif
 		switch (ch) {
 		case 'A':
@@ -370,11 +369,6 @@ main(int argc, char *argv[])
 		case 'd':
 			tmpdir = optarg;
 			break;
-		case 'E':	
-			ex_timeout = (int)strtonum(optarg, 1LL, MAX_TIMEOUT, &errstr);
-			if (errstr)
-				errx(1, "timeout is %s: %s", errstr, optarg);
-			break;
 		case 'e':
 			logfile = optarg;
 			break;
@@ -532,10 +526,6 @@ main(int argc, char *argv[])
 			printf("maxprocs: %lld\n", (long long)rlimit_nproc);
 		else
 			printf("maxprocs: not set\n");
-		if (ex_timeout != -1)
-			printf("timeout: %d\n", ex_timeout);
-		else
-			printf("timeout: not set\n");
 #ifdef ENABLE_LDAP
 		printf("ldapurl: %s\n", ldapurl != NULL ? ldapurl : "not set");
 		printf("ldaphost: %s\n", ldaphost);
@@ -648,8 +638,7 @@ main(int argc, char *argv[])
 		if ((be[n].be_name = dlsym(be[n].dlhandle, "vilter_name")) == NULL)
 			errx(1, "no function vilter_name in backend %s", be[n].name);
 
-		be[n].be_new = dlsym(be[n].dlhandle, "vilter_new");	/* Optional */
-		be[n].be_end = dlsym(be[n].dlhandle, "vilter_end");	/* Optional */
+		be[n].be_envfrom = dlsym(be[n].dlhandle, "vilter_envfrom");	/* Optional */
 		
 		if ((be[n].be_type = dlsym(be[n].dlhandle, "vilter_type")) == NULL)
 			errx(1, "no function vilter_type in backend %s", be[n].name);
@@ -682,57 +671,61 @@ main(int argc, char *argv[])
 	/* Analye a signe file if requested by the user, else normal operation */
 
 	if (fnam != NULL) {
-		struct be_data *data;
-		char		scanfn[PATH_MAX];
-		char		virus[MAXLEN];
-		char		reason[MAXLEN];
-		double		score, threshold;
-		int		scanresult;
+		char		 scanfn[PATH_MAX];
+		char		 virus[MAXLEN];
+		char		 reason[MAXLEN];
+		double		 score, threshold;
+		int		 scanresult;
+		int		 exval;
 
-		data = malloc(sizeof(struct be_data) * nbe);
+		exval = 0;
 		for (n = 0; n < nbe; n++) {
 			scanresult = SCAN_OK;
 			if (be[n].type & BE_SCAN_VIRUS) {
-				warnx("scanning %s for viruses using backend %s", fnam, be[n].name);
+				if (verbose)
+					warnx("scanning %s for viruses using backend %s", fnam, be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
 				bzero(virus, sizeof(virus));
-				scanresult = be[n].be_scan.be_scan_virus(&data[n], scanfn, NULL, virus, sizeof(virus));
-
+				scanresult = be[n].be_scan.be_scan_virus(scanfn, sizeof(scanfn), NULL, virus, sizeof(virus));
 			} else if (be[n].type & BE_SCAN_SPAM) {
-				warnx("scanning %s for spam using backend %s", fnam, be[n].name);
+				if (verbose)
+					warnx("scanning %s for spam using backend %s", fnam, be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
-				scanresult = be[n].be_scan.be_scan_spam(&data[n], scanfn, NULL, &score, &threshold);
-
+				scanresult = be[n].be_scan.be_scan_spam(scanfn, sizeof(scanfn), NULL, &score, &threshold);
 			} else if (be[n].type & BE_SCAN_UNWANTED) {
-				warnx("scanning %s for unwanted content using backend %s", fnam, be[n].name);
+				if (verbose)
+					warnx("scanning %s for unwanted content using backend %s", fnam, be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
 				bzero(reason, sizeof(reason));
-				scanresult = be[n].be_scan.be_scan_unwanted(&data[n], scanfn, NULL, reason, sizeof(reason));
+				scanresult = be[n].be_scan.be_scan_unwanted(scanfn, sizeof(scanfn), NULL, reason, sizeof(reason));
 			}
 			switch (scanresult) {
 			case SCAN_OK:
-				warnx("file scanned ok");
+				printf("file scanned ok\n");
 				break;
 			case SCAN_ERROR:
-				warnx("error during scan");
+				printf("error during scan\n");
+				exval = 1;
 				break;
 			case SCAN_VIRUS:
-				warnx("virus '%s' detected in file", virus);
+				printf("virus '%s' detected in file\n", virus);
+				exval = 1;
 				break;
 			case SCAN_SPAM:
-				warnx("classified as spam, score = %f, threshold %f", score, threshold);
+				printf("classified as spam, score = %f, threshold %f\n", score, threshold);
+				exval = 1;
 				break;
 			case SCAN_UNWANTED:
-				warnx("unwanted content, reson = %s", reason);
+				printf("unwanted content, reson = %s\n", reason);
+				exval = 1;
 				break;
 			}
 		}
 
-		free(data);
 #ifdef ENABLE_LDAP
 		ldap_unbind(ld);
 #endif
-		return 0;
+		return exval;
 	}
 
 	uid = getuid();
@@ -815,7 +808,7 @@ main(int argc, char *argv[])
 				}
 			} else if (nfds > 0 && pfd[0].revents & POLLIN) {
 				/* nfds --; */
-				if (dispatch_imsg(ibuf_e, 0) == -1)
+				if (!child_quit && dispatch_imsg(ibuf_e, 0) == -1)
 					quit = 1;
 			}
 
@@ -823,14 +816,15 @@ main(int argc, char *argv[])
 			if (pftable_timeout())
 				syslog(LOG_WARNING, "can not timeout pf tables");
 #endif
-			if (child_quit)
+			if (child_quit && verbose)
 				warnx("child process terminated");
 		}
 
 		if (quit && !child_quit) {	/* XXX is this necessary? */
 			sleep(1);
 			if (!child_quit) {
-				warnx("signalling child process to terminate");
+				if (verbose)
+					warnx("signalling child process to terminate");
 				kill(child_pid, SIGTERM);
 			}
 		}
